package com.saa.bondoapp.fragment.scan

import android.Manifest
import android.annotation.SuppressLint
import android.app.Activity
import android.content.Intent
import android.graphics.Bitmap
import android.graphics.ImageDecoder
import android.os.Build
import android.os.Bundle
import android.provider.MediaStore
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.activity.result.ActivityResultLauncher
import androidx.activity.result.contract.ActivityResultContracts
import androidx.fragment.app.Fragment
import androidx.lifecycle.ViewModelProvider
import androidx.lifecycle.lifecycleScope
import com.saa.bondoapp.data.db.AppDatabase
import com.saa.bondoapp.data.db.entities.Transaction
import com.saa.bondoapp.data.repositories.TransactionRepository
import com.saa.bondoapp.databinding.FragmentScanBinding
import com.saa.bondoapp.util.LocationContext
import com.saa.bondoapp.util.fitBitmapToView
import com.saa.bondoapp.util.toast
import java.io.IOException

class ScanFragment : Fragment(), ScanListener {

    private var _binding: FragmentScanBinding? = null
    private val binding get() = _binding!!

    private lateinit var captureImageLauncher: ActivityResultLauncher<Intent>
    private lateinit var pickImageLauncher: ActivityResultLauncher<Intent>

    private lateinit var submitTransactionsSheet: SubmitTransactionsFragment

    private val locationContext by lazy { LocationContext(this) }


    @SuppressLint("NewApi")
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        captureImageLauncher = registerForActivityResult(ActivityResultContracts.StartActivityForResult()) { result ->
            if (result.resultCode == Activity.RESULT_OK) {
                val data: Intent? = result.data

                var image: Bitmap?

                if (Build.VERSION.SDK_INT >= 33) { // TIRAMISU
                    image = data?.getParcelableExtra("data", Bitmap::class.java)
                } else {
                    image = data?.getParcelableExtra<Bitmap>("data")
                }

                binding.viewmodel?.image = image!!
                displayImagePreview(image)
            }
        }


        pickImageLauncher = registerForActivityResult(ActivityResultContracts.StartActivityForResult()) { result ->
            try {
                val source = ImageDecoder.createSource(activity?.contentResolver!!, result.data?.data!!)
                val image = ImageDecoder.decodeBitmap(source)

                binding.viewmodel?.image = image
                displayImagePreview(image)
            } catch (e: IOException) {
                // Handle any exceptions (e.g., file not found, permission issues, etc.)
            }
        }

        submitTransactionsSheet = SubmitTransactionsFragment()
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View? {
        _binding = FragmentScanBinding.inflate(inflater, container, false);
        val view = binding.root

        askPermission()

        val db = AppDatabase(requireActivity())
        val repository = TransactionRepository(db)
        val factory = ScanViewModelFactory(repository)

        val viewModel = ViewModelProvider(this, factory).get(ScanViewModel::class.java)
        binding.viewmodel = viewModel
        viewModel.scanListener = this

        binding.captureImageButton.setOnClickListener {
            val intent = Intent(MediaStore.ACTION_IMAGE_CAPTURE)
            captureImageLauncher.launch(intent)
        }

        binding.pickImageButton.setOnClickListener {
            val intent = Intent(Intent.ACTION_GET_CONTENT)
            intent.type = "image/*" // Specify the MIME type for images
            pickImageLauncher.launch(intent)
        }


        locationContext.askPermission()
        locationContext.updateLocation()

        viewModel.location = locationContext.location
        viewModel.locality = locationContext.locality
        return view
    }

    fun displayImagePreview(image: Bitmap) {
        val originalBitmap: Bitmap = image
        val scaledBitmap = fitBitmapToView(binding.picturePreviewView, originalBitmap)
        binding.picturePreviewView.setImageBitmap(scaledBitmap)

    }

    override fun onSuccess(transactions: List<Transaction>) {
        val countTransactions = transactions.size
        toast(requireActivity(), "Fetched $countTransactions transactions!")
        submitTransactionsSheet._transactions = transactions
        submitTransactionsSheet.show(parentFragmentManager,"Submit sheet")
    }

    override fun onFailure(message: String) {
        toast(requireActivity(), message)
    }

    fun askPermission() {
        val permissionLauncher = registerForActivityResult(
            ActivityResultContracts.RequestPermission()
        ) { isGranted ->
            if (isGranted) {
                //
            } else {
                toast(requireActivity(), "Please grant the camera access!")
            }
        }
        permissionLauncher.launch(Manifest.permission.CAMERA)
    }

}