package com.saa.bondoapp.fragment.settings

import android.app.AlertDialog
import android.content.Intent
import android.os.Bundle
import android.os.Environment
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.core.content.FileProvider
import androidx.fragment.app.Fragment
import androidx.localbroadcastmanager.content.LocalBroadcastManager
import com.saa.bondoapp.data.db.AppDatabase
import com.saa.bondoapp.data.db.entities.CURRENT_USER_NIM
import com.saa.bondoapp.data.repositories.TransactionRepository
import com.saa.bondoapp.databinding.FragmentSettingsBinding
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext
import org.apache.poi.hssf.usermodel.HSSFWorkbook
import org.apache.poi.xssf.usermodel.XSSFWorkbook
import java.io.File
import java.io.FileOutputStream
import java.time.LocalDateTime
import java.time.format.DateTimeFormatter
import kotlin.random.Random

class SettingsFragment : Fragment() {

    private lateinit var binding: FragmentSettingsBinding
    private val db by lazy { AppDatabase(requireActivity()) }
    private val repository by lazy { TransactionRepository(db) }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        binding = FragmentSettingsBinding.inflate(inflater, container, false)

        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        binding.randomizeBtn.setOnClickListener {
            context?.let { it1 ->
                LocalBroadcastManager.getInstance(it1)
                    .sendBroadcast(Intent("com.saa.bondoapp.RANDOMIZE").apply {
                        putExtra("nominal", Random.nextLong(0, 20000000L))
                    })
            }
        }

        binding.exportTransactionsBtn.setOnClickListener {
            context?.let {
                showExportFormatPopup(email = false)
            }
        }

        binding.emailTransactionsBtn.setOnClickListener {
            context?.let {
                showExportFormatPopup(email = true)
            }
        }

        binding.logoutBtn.setOnClickListener {
            context?.let { it1 ->
                LocalBroadcastManager.getInstance(it1)
                    .sendBroadcast(Intent("com.saa.bondoapp.LOGOUT"))
            }
        }

    }

    private fun showExportFormatPopup(email: Boolean) {
        val builder = AlertDialog.Builder(requireContext())
        builder.setTitle("Select Export Format")
        val formats = arrayOf(".xls", ".xlsx")
        builder.setItems(formats) { _, which ->
            when (which) {
                0 -> {
                    CoroutineScope(Dispatchers.IO).launch {
                        Log.d("SHEET", "Chosen xls...")
                        val spreadsheetFile = exportSpreadsheet("xls")

                        if (email) {
                            sendEmail(spreadsheetFile, "${CURRENT_USER_NIM}.std.stei.itb.ac.id")
                        }
                    }
                }
                1 -> {
                    CoroutineScope(Dispatchers.IO).launch {
                        Log.d("SHEET", "Chosen xlsx...")
                        val spreadsheetFile = exportSpreadsheet("xlsx")

                        if (email) {
                            sendEmail(spreadsheetFile, "${CURRENT_USER_NIM}.std.stei.itb.ac.id")
                        }
                    }
                }
            }
        }
        builder.show()
    }

    private suspend fun exportSpreadsheet(format: String): File {
        return withContext(Dispatchers.IO) {
            showToast("Generating spreadsheet...")
            Log.d("SHEET", "Generating spreadsheet...")

            val workbook = when (format) {
                "xls" -> HSSFWorkbook()
                "xlsx" -> XSSFWorkbook()
                else -> throw IllegalArgumentException("Unsupported format: $format")
            }

            val sheet = workbook.createSheet("Transactions")

            val headerRow = sheet.createRow(0)
            headerRow.createCell(0).setCellValue("Date")
            headerRow.createCell(1).setCellValue("Name")
            headerRow.createCell(2).setCellValue("Category")
            headerRow.createCell(3).setCellValue("Nominal")
            headerRow.createCell(4).setCellValue("Location")
            headerRow.createCell(5).setCellValue("Longitude")
            headerRow.createCell(6).setCellValue("Latitude")

            val transactions = repository.getTransactions()

            transactions.forEachIndexed { index, t ->
                val row = sheet.createRow(index + 1)
                row.createCell(0).setCellValue(t.date ?: "")
                row.createCell(1).setCellValue(t.title ?: "")
                row.createCell(2).setCellValue(t.category ?: "")
                row.createCell(3).setCellValue(t.nominal?.toDouble() ?: 0.0)
                row.createCell(4).setCellValue(t.locality ?: "")
                row.createCell(5).setCellValue(t.location_long ?: 0.0)
                row.createCell(6).setCellValue(t.location_lat ?: 0.0)
            }

            val currentDateTime = LocalDateTime.now()
            val formatter = DateTimeFormatter.ofPattern("yyyyMMddHHmmss")
            val datetime = currentDateTime.format(formatter)

            val file = File(
                Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS),
                "Transactions$datetime.$format"
            )

            val fileOutputStream = FileOutputStream(file)
            workbook.write(fileOutputStream)
            fileOutputStream.close()

            showToast("Spreadsheet saved successfully at: ${file.absolutePath}")
            Log.d("SHEET", "Spreadsheet saved successfully at: ${file.absolutePath}")

            file
        }
    }

    private fun sendEmail(file: File, emailAddress: String) {
        val emailIntent = Intent(Intent.ACTION_SEND)
        emailIntent.type = "text/plain"
        emailIntent.putExtra(Intent.EXTRA_EMAIL, arrayOf(emailAddress))
        emailIntent.putExtra(Intent.EXTRA_SUBJECT, "Transactions Data")
        emailIntent.putExtra(Intent.EXTRA_TEXT, "Please find attached your transaction data. Thank you.")

        val uri = FileProvider.getUriForFile(requireContext(), "com.saa.bondoapp.fileprovider", file)
        emailIntent.putExtra(Intent.EXTRA_STREAM, uri)
        emailIntent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION)

        try {
            startActivity(Intent.createChooser(emailIntent, "Choose email client..."))
        } catch (e: Exception) {
            showToast("Error when trying to send email.")
        }
    }

    private fun showToast(message: String) {
        requireActivity().runOnUiThread {
            Toast.makeText(requireContext(), message, Toast.LENGTH_SHORT).show()
        }
    }
}